/*
  Doc_Arduino - german Arduino Forum
  Arduino IDE 1.8.15
  Arduino Nano Every
  01.08.2021
  License: GNU GPLv3
  
  Komplexeres Beispiel mit der CCL – Configurable Custom Logic    
  kombiniert mit dem EVESYS - Eventsystem.                                                                                                      
*/
/*
  Arduino Nano Every PinOut (ATmega4809)
                           _
                        __|_|__    
                       |       |  
      (SCK)(D13)  PE2  +       +  PE1  (D12)(MISO)  
                 3.3V  +       +  PE0  (D11)(MOSI)  
           (AREF) PD7  +       +  PB1  (D10)(~)(TCA0 Channel 1)
       (A0)(D14)  PD3  +       +  PB0  (D 9)(~)(TCA0 Channel 0)
       (A1)(D15)  PD2  +       +  PE3  (D 8)(SS)
       (A2)(D16)  PD1  +       +  PA1  (D 7) 
       (A3)(D17)  PD0  +       +  PF4  (D 6)(~)(TCB0)
  (SDA)(A4)(D18)  PF2  +       +  PB2  (D 5)(~)(TCA0 Channel 2)
  (SDL)(A5)(D19)  PF3  +       +  PC6  (D 4)  
       (A6)(D20)  PD4  +       +  PF5  (D 3)(~)(TCB1)  
       (A7)(D21)  PD5  +       +  PA0  (D 2) 
                 5.0V  +       +  GND
                RESET  +       +  RESET
                  GND  +       +  PC5  (D 0) RX1
                  VIN  +       +  PC4  (D 1) TX1      
                       |_______|  

//----------------------------------------------------------------------------------------------------------------------------------------------
!!! Arduino Nano Every with ATmega4809 !!!

 With 'default' in the table the view from the µC is meant. Not the one of the Arduino framework. 
 If you don't change anything at the portmux registers, the setting according to the board label above is valid.

 Ardu      |    Event       | EVSYS  |      CCL-LUTn     |       TCA0        |       TCBn        |  ADC0   |
  Pin Port | Port | Channel |        | default | PortMux | default | PortMux | default | PortMux | default |
   0  PC5  |  0     2 or 3  |  ----  |   ---       ---   |   ---      0-WO5  |   ---       ---   |         |
   1  PC4  |  0     2 or 3  |  ----  |   ---       ---   |   ---      0-WO4  |   ---       ---   |         |
   2  PA0  |  0     0 or 1  |  ----  |  0-IN0      ---   |  0-WO0      ---   |   ---       ---   |         |
   3  PF5  |  1     4 or 5  |  ----  |   ---       ---   |   ---      0-WO5  |   ---      1-WO   |  AIN15  |
   4  PC6  |  0     2 or 3  |  ----  |   ---      1-OUT  |   ---       ---   |   ---       ---   |         |
   5  PB2  |  1     0 or 1  | EVOUTB |   ---       ---   |   ---      0-WO2  |   ---       ---   |         |
   6  PF4  |  1     4 or 5  |  ----  |   ---       ---   |   ---      0-WO4  |   ---      0-WO   |  AIN14  |
   7  PA1  |  0     0 or 1  |  ----  |  0-IN1      ---   |  0-WO1      ---   |   ---       ---   |         |
   8  PE3  |  0     4 or 5  |  ----  |   ---       ---   |   ---      0-WO3  |   ---       ---   |  AIN11  |
   9  PB0  |  1     0 or 1  |  ----  |   ---       ---   |   ---      0-WO0  |   ---       ---   |         |
  10  PB1  |  1     0 or 1  |  ----  |   ---       ---   |   ---      0-WO1  |   ---       ---   |         |
  11  PE0  |  0     4 or 5  |  ----  |   ---       ---   |   ---      0-WO0  |   ---       ---   |  AIN8   |
  12  PE1  |  0     4 or 5  |  ----  |   ---       ---   |   ---      0-WO1  |   ---       ---   |  AIN9   |
  13  PE2  |  0     4 or 5  | EVOUTE |   ---       ---   |   ---      0-WO2  |   ---       ---   |  AIN10  |
  14  PD3  |  1     2 or 3  |  ----  |  2-OUT      ---   |   ---      0-WO3  |   ---       ---   |  AIN3   |
  15  PD2  |  1     2 or 3  | EVOUTD |  2-IN2      ---   |   ---      0-WO2  |   ---       ---   |  AIN2   |
  16  PD1  |  1     2 or 3  |  ----  |  2-IN1      ---   |   ---      0-WO1  |   ---       ---   |  AIN1   |
  17  PD0  |  1     2 or 3  |  ----  |  2-IN0      ---   |   ---      0-WO0  |   ---       ---   |  AIN0   |
  18  PF2  |  1     4 or 5  | EVOUTF |  3-IN2      ---   |   ---      0-WO2  |   ---       ---   |  AIN12  |  board internal double assignment with PA2 observe !!! (EVOUTA)
  19  PF3  |  1     4 or 5  |  ----  |  3-OUT      ---   |   ---      0-WO3  |   ---       ---   |  AIN13  |  board internal double assignment with PA3 observe !!!
  20  PD4  |  1     2 or 3  |  ----  |   ---       ---   |   ---      0-WO4  |   ---       ---   |  AIN4   |
  21  PD5  |  1     2 or 3  |  ----  |   ---       ---   |   ---      0-WO5  |   ---       ---   |  AIN5   |

                                                           ____
   (4)(PC6)(Event)>---+------------------------>(EventA)--|    |
    CW Freigabe       |                                   | &  |---> (CW)(15)(PD2)(EVOUTD) --> BTS (R_PWM)     
                      |                     +-->(EventB)--|____|
                      |             ____    |              LUT1 
                      +--(EventA)--|    |   |
   (2)(PA0)(0-IN0)>----------------|    |---+ 
    PWM Takteingang   +--(EventB)--|____|   |
                      |             LUT0    |                             
                      |                     |              ____
                      |                     +-->(EventA)--|    |
                      |                                   | &  |---> (CCW)(14)(PD3)(2-OUT) --> BTS (L_PWM) 
   (7)(PA1)(Event)>---+------------------------>(EventB)--|____|    
    CCW Freigabe                                           LUT2                                                                                                            

 Ardu      |    Event       | EVSYS  | CCL-LUTn |
  Pin Port | Port | Channel |        |          |
   2  PA0  |                |        |  0-IN0   | PWM-IN  
   4  PC6  |  0     2 or 3  |        |          | Freigabe CW
   7  PA1  |  0     0 or 1  |        |          | Freigabe CCW    
  15  PD2  |                | EVOUTD |          | PWM-OUT  CW
  14  PD3  |                |        |  2-OUT   | PWM-OUT  CCW
  
  Wahrheitstabelle:
  IN2 | IN1 | IN0 | LUTnOUT [bit number in CCL.TRUTHn]
   0  |  0  |  0  | TRUTHn[0]
   0  |  0  |  1  | TRUTHn[1]
   0  |  1  |  0  | TRUTHn[2]
   0  |  1  |  1  | TRUTHn[3]
   1  |  0  |  0  | TRUTHn[4]
   1  |  0  |  1  | TRUTHn[5]
   1  |  1  |  0  | TRUTHn[6]
   1  |  1  |  1  | TRUTHn[7]                   
*/

#include <NanoEveryPin.h>
#include <NanoEveryPinNamen.h>

InputPin  <pinPA0> pinInPWM;
InputPin  <pinPC6> pinEnableCW;   
InputPin  <pinPA1> pinEnableCCW;
OutputPin <pinPD2> pinOutCW; 
OutputPin <pinPD3> pinOutCCW;
OutputPin <LED_BUILTIN> onboardLed;

void setup (void)
{
  pinEnableCW.init();   pinEnableCW.enablePullup();   pinEnableCW.enableInvertIO();
  pinEnableCCW.init();  pinEnableCCW.enablePullup();  pinEnableCCW.enableInvertIO();
  pinInPWM.init();      pinInPWM.enablePullup();      
  pinOutCW.init(); 
  pinOutCCW.init(); 
  onboardLed.init();    
  
  initEventSystem();
  initLUT2CCL();
  initLUT1CCL();
  initLUT0CCL();
  CCL.CTRLA = CCL_ENABLE_bm;           // general enable CCL module
}

void loop (void)
{
  heartbeat(500);
}

void initEventSystem(void)
{
  // Enable-CW auf 2 Eingänge verteilen
  EVSYS.CHANNEL2 = EVSYS_GENERATOR_PORT0_PIN6_gc;   // PC6 
  EVSYS.USERCCLLUT1A = EVSYS_CHANNEL_CHANNEL2_gc;   // 
  EVSYS.USERCCLLUT0A = EVSYS_CHANNEL_CHANNEL2_gc;   // 
  
  // Enable-CCW auf 2 Eingänge verteilen
  EVSYS.CHANNEL1 = EVSYS_GENERATOR_PORT0_PIN1_gc;   // PA1
  EVSYS.USERCCLLUT0B = EVSYS_CHANNEL_CHANNEL1_gc;   // 
  EVSYS.USERCCLLUT2B = EVSYS_CHANNEL_CHANNEL1_gc;   // 
  
  // LUT0-OUT auf 2 Eingänge verteilen
  EVSYS.CHANNEL0 = EVSYS_GENERATOR_CCL_LUT0_gc;     // 
  EVSYS.USERCCLLUT1B = EVSYS_CHANNEL_CHANNEL0_gc;   // 
  EVSYS.USERCCLLUT2A = EVSYS_CHANNEL_CHANNEL0_gc;   //

  // LUT1-OUT zum Pin rausführen
  EVSYS.CHANNEL3 = EVSYS_GENERATOR_CCL_LUT1_gc;     // 
  EVSYS.USEREVOUTD = EVSYS_CHANNEL_CHANNEL3_gc;     // PD2

  // LUT2-OUT geht direkt zum Pin                   // PD3
}

void initLUT2CCL(void)
{
   CCL.LUT2CTRLB  = CCL_INSEL0_EVENTA_gc; // LUT2-IN0 input source from Event
   CCL.LUT2CTRLB |= CCL_INSEL1_EVENTB_gc; // LUT2-IN1 input source from Event
   
   CCL.TRUTH2 = _BV(3);                   // Configure Truth Table, AND  
   
   CCL.LUT2CTRLA  = CCL_OUTEN_bm;         // enable Output LUT2
   CCL.LUT2CTRLA |= CCL_ENABLE_bm;        // enable LUT2
}

void initLUT1CCL(void)
{
   CCL.LUT1CTRLB  = CCL_INSEL0_EVENTA_gc; // LUT1-IN0 input source from Event
   CCL.LUT1CTRLB |= CCL_INSEL1_EVENTB_gc; // LUT1-IN1 input source from Event
   
   CCL.TRUTH1 = _BV(3);                   // Configure Truth Table, AND 
   
   CCL.LUT1CTRLA |= CCL_ENABLE_bm;        // enable LUT1
}

void initLUT0CCL(void)
{
  CCL.LUT0CTRLB  = CCL_INSEL0_IO_gc;      // LUT0-IN0 input pin
  CCL.LUT0CTRLB |= CCL_INSEL1_EVENTA_gc;  // LUT0-IN1 input source from Event 
  CCL.LUT0CTRLC  = CCL_INSEL2_EVENTB_gc;  // LUT0-IN2 input source from Event
   
  CCL.TRUTH0 = _BV(5) | _BV(3);           // Configure Truth Table, (IN0&IN1) XOR (IN0&IN2)

  CCL.LUT0CTRLA |= CCL_ENABLE_bm;         // enable LUT0
}

void heartbeat (const unsigned long interval)                     
{  
  static unsigned long lastMillis {0};
    
  if (millis() - lastMillis >= interval) {
    lastMillis += interval;
    onboardLed.toggle();
  }
}
